#include "socketudp.h"
#include "socketexceptions.h"

CSocketUDP::CSocketUDP(const std::string &sock_id) : CSocket(sock_id,socket_udp)
{
  this->associated_port=-1;
  this->clients.clear();
}

void CSocketUDP::hard_open(void *comm_dev)
{
  int *port=(int *)comm_dev;
  sockaddr_in sock;

  CSocket::hard_open(NULL);
  if(port!=NULL)
  {
    memset(&sock,0,sizeof(sock));
    sock.sin_family=AF_INET;
    sock.sin_addr.s_addr=htonl(INADDR_ANY);
    sock.sin_port=htons(*port);
    // bind to the desired IP and port to receive data from
    if((::bind(this->socket_fd,(struct sockaddr *)&sock,sizeof(sock)))<0)
    {
      /* handle exceptions */
      throw CSocketException(_HERE_, "Error with connect function", this->comm_id);
    }
    else
      this->associated_port=*port;
  }
}

int CSocketUDP::hard_read(unsigned char *data, int len)
{
  CEventServer *event_server=CEventServer::instance();
  unsigned int i=0;
  struct sockaddr_in sock;
  socklen_t sock_len=sizeof(sock);
  int read=0;

  sock.sin_family=AF_INET;
  if((read=recvfrom(this->socket_fd,data,len,0,(struct sockaddr *)&sock,&sock_len))<0)
  {
    /* handle exceptions */
    throw CSocketException(_HERE_,"Impossible to read from the socket",this->comm_id);
  }
  // enqueue the data to the corresponding client
  for(i=0;i<this->clients.size();i++)
    if(this->clients[i]->ip_address==std::string(inet_ntoa(sock.sin_addr)))
    {
      this->clients[i]->data_queue.write(data,len);
      if(!event_server->event_is_set(this->clients[i]->new_data_avail_event_id))
        event_server->set_event(this->clients[i]->new_data_avail_event_id);
    }

  return read;
}

void CSocketUDP::hard_close(void)
{
  CEventServer *event_server=CEventServer::instance();
  unsigned int i;

  CSocket::hard_close();
  this->associated_port=-1;
  for(i=0;i<this->clients.size();i++)
  {
    event_server->delete_event(this->clients[i]->new_data_avail_event_id);
    this->clients[i]->new_data_avail_event_id="";
    this->clients[i]->ip_address="";
  }
  this->clients.clear();
}

int CSocketUDP::write_to(std::string &ip,int port,unsigned char *data,int len)
{
  sockaddr_in sock;
  int written=0;

  memset(&sock,0,sizeof(sock));
  sock.sin_family=AF_INET;
  if(inet_aton(ip.c_str(),&sock.sin_addr)==0)
  {
    /* handle exceptions */
    throw CSocketException(_HERE_,"Impossible to convert IP address",this->comm_id);
  }
  else
  {
    sock.sin_port=htons(port);
    if((written=sendto(this->socket_fd,data,len,0,(struct sockaddr *)&sock,sizeof(sock)))<0)
    {
      /* handle exceptions */
      throw CSocketException(_HERE_,"Impossible to send data to the desired IP and port",this->comm_id);
    }
    return written;
  }
}

int CSocketUDP::read_from(std::string &ip, unsigned char *data, int len)
{
  unsigned int i=0;
  int read=0;
  
  for(i=0;i<this->clients.size();i++)
  {
    if(this->clients[i]->ip_address==ip)
    {
      read=this->clients[i]->data_queue.read(data,len);
      return read;
    }
  }
  // handle exceptions
  throw CSocketException(_HERE_,"The given IP address is not in the allowed list",this->comm_id);
}

int CSocketUDP::get_num_data_from(std::string &ip)
{
  unsigned int i=0;

  for(i=0;i<this->clients.size();i++)
  {
    if(this->clients[i]->ip_address==ip)
    {
      return this->clients[i]->data_queue.get_num_data();
    }
  }
  // handle exceptions
  throw CSocketException(_HERE_,"The given IP address is not in the allowed list",this->comm_id);
}

std::string CSocketUDP::add_client(std::string &ip)
{
  CEventServer *event_server=CEventServer::instance();
  TUDPClient *new_client=new TUDPClient;
  unsigned int i=0;

  for(i=0;i<this->clients.size();i++)
  {
    if(this->clients[i]->ip_address==ip)
    {
      /* handle exceptions */
      throw CSocketException(_HERE_,"The given IP addres is alreasy accepted",this->comm_id);
    }
  }
  // add the new IP address to wait data for
  new_client->ip_address=ip;
  new_client->new_data_avail_event_id=ip + "_data_avail_event_id";
  event_server->create_event(new_client->new_data_avail_event_id);
  this->clients.push_back(new_client);

  return new_client->new_data_avail_event_id; 
}

CSocketUDP::~CSocketUDP()
{

}
